open Json_io
open Json_type

(* {"error":"file_exists","reason":"The database could not be created,
   the file already exists."} *)
(* {"error":"not_found","reason":"no_db_file"} *)
(* {"error":"conflict","reason":"Document update conflict."} *)
type json error = { error: string; reason: string }

(* {"couchdb":"Welcome","version":"0.10.1"} *)
type json version = { couchdb: string; version: string }

(* {“ok”:true} *)
(* {"ok":true,"id":"ec06f2bb78b41d7275c0ea8a33ce214a","rev":"1-2098454642"} *)
type json ok = { ok: bool; ?id: string = ""; ?rev: string = "" }

(* {"language": "javascript", *)
(* "views" : { *)
(*   "posts": { *)
(*     "map": "function (doc) { if ( doc.type && doc.type == \"blog post\" ) { emit ( null , doc ); } }" *)
(*   } *)
(* }} *)
type json view = { map: string; ?reduce: string option }
and design = { language: string; views: (string * view) assoc }

let create_view ?(reduce="") name ~map =
  name, {map = map; reduce = if reduce = "" then None else (Some reduce) }

(* {"db_name":"ocsigen","doc_count":1,"doc_del_count":2,"update_seq":5,"purge_seq":0,"compact_running":false,"disk_size":20569,"instance_start_time":"1266315084857335","disk_format_version":4} *)
type json infos = { db_name: string; doc_count: int; doc_del_count: int; update_seq: int; purge_seq: int; compact_running: bool; disk_size: int; instance_start_time: string; disk_format_version: int }

module Parser =
struct
  let version ans =
    let parsed = json_of_string ans in
    let ans = version_of_json parsed in
      if ans.couchdb = "Welcome" then
        Some ans.version
      else
        None
  let infos ans =
    let parsed = json_of_string ans in
      infos_of_json parsed
  let ok ans =
    let parsed = json_of_string ans in
    let ans = ok_of_json parsed in
      ans.ok
  let id_rev ans =
    let parsed = json_of_string ans in
    let ans = ok_of_json parsed in
      ans.id, ans.rev
  let error ans =
    let parsed = json_of_string ans in
      error_of_json parsed
  let view = function
    | Object [ "total_rows", Int total_rows;
               "offset", Int offset;
               "rows", Array rows ] -> total_rows, rows
    | _ -> 0, []
  let value val_of_json = function
    | Object [ "id", String id;
               "key", String key;
               "value", value ] -> val_of_json value
    | Object [ "id", String id;
               "key", Null ;
               "value", value ] -> val_of_json value
    | _ -> failwith "Not a good answer"
end

